--
--  PICA-Style Output for Class B
--  (combinational-output Moore)
--  Low Pass Filter FSM Example
--  Tom Mayo - 6-15-95
--

ENTITY lpf IS
  PORT ( clk : IN bit;
    rstn : IN bit;
    ena : IN bit;
    din : IN bit;
    dout : OUT bit
  );
END lpf;

ARCHITECTURE pica_b OF lpf IS
  SIGNAL present_state, next_state : bitvec;
  LABEL registers, transitions;
BEGIN

  registers : PROCESS ( clk, rstn )
  BEGIN
    IF ( rstn = '0' ) THEN
      present_state <= "00"; -- s0
    ELSIF ( clk'RISING ) THEN
      present_state <= next_state;
    END IF;
  END PROCESS;

  transitions : PROCESS ( present_state, ena, din )
  BEGIN
    CASE present_state IS
      WHEN "00" => -- s0
        dout <= '0';
        IF ( ena = '1' AND din = '1' ) THEN
          next_state <= "01"; -- s0x
        ELSE
          next_state <= "00"; -- s0
        END IF;
      WHEN "01" => -- s0x
        dout <= '0';
        IF ( ena = '1' AND din = '1' ) THEN
          next_state <= "10"; -- s1
        ELSIF ( ena = '1' AND din = '0' ) THEN
          next_state <= "00"; -- s0
        ELSE
          next_state <= "01"; -- s0x
        END IF;
      WHEN "10" => -- s1
        dout <= '1';
        IF ( ena = '1' AND din = '0' ) THEN
          next_state <= "11"; -- s1x
        ELSE
          next_state <= "10"; -- s1
        END IF;
      WHEN "11" => -- s1x
        dout <= '1';
        IF ( ena = '1' AND din = '0' ) THEN
          next_state <= "00"; -- s0
        ELSIF ( ena = '1' AND din = '1' ) THEN
          next_state <= "10"; -- s1
        ELSE
          next_state <= "11"; -- s1x
        END IF;
    END CASE;
  END PROCESS;
END pica_b;
